{
                        
  JAM_FastWriteDemo_011

  CHANGES: Uses setting the counter output pin
  rather than DIRA to control whether clock strobe
  is as output or not 

  WRITES A LONG-BUFFER FROM COG RAM INTO
  SRAM AT 15MHz FREQUENCY (=15 000 000 BYTES
  PER SECOND) IN ASSEMBLER.

  This demo writes the buffer a hundred times - the demo will
  take 10 seconds to run, with no output until it's done.
  Most of this time is taken by using the SPIN functions to
  verify the written data, to start the cog every time etc.,
  the actual memory writing is... fast!

  When the demo finishes, it displays the SRAM contents (first
  256 bytes plus some extra to indicate that only 256 bytes was
  written).  

  How it works:

  The demo uses the video generator in VGA mode, ie. it outputs
  raw bytes using the video generator. The CTRA, which is used
  to generate the clock for the video generator is also used
  to generate the clock strobe for the SRAM card (yes, the
  CTRA may be in any one of the first three (PLL) modes and
  work with the video generator). One waitvid-instruction
  outputs 4 bytes.  
      

  /-------------------------------------------------------\
  |                                                       |
  |  Copyright Jasper Mattsson 2007                       |
  |  May be freely distributed.                           |                                                 
  |                                                       |
  |  Contact me:                                          |
  |  Parallax or XGameStation forums: Jasper_M            |
  |  Email/MSN: jasper.mattsson@gmail.com                 |
  |                                                       |
  \-------------------------------------------------------/

  
}
CON

  _clkmode = xtal1 + pll8x
  _xinfreq = 10_000_000

  'CON BLOCK STOLEN FROM THE ACTUAL DRIVER ^___^ 
 
  ' SRAM bus interface pin constants
  SRAM_CTRL_0 = 1 ' NET_RX_CLK       (expansion pin 10)
  SRAM_CTRL_1 = 2 ' NET_TX_DATA      (expansion pin 9)

  SRAM_STROBE = 30 ' USB_RXD (Prop TX ----> USB_RXD Host) (expansion pin 19)

  SRAM_IO_7 = 23 ' IO_7 (pin 28)
  SRAM_IO_6 = 22
  SRAM_IO_5 = 21
  SRAM_IO_4 = 20
  SRAM_IO_3 = 19
  SRAM_IO_2 = 18
  SRAM_IO_1 = 17
  SRAM_IO_0 = 16  ' IO_0 (pin 21)

    ' sram commands
  SRAM_CMD_WRITE   = %000000_00
  SRAM_CMD_READ    = %000000_01
  SRAM_CMD_LOADLO  = %000000_10
  SRAM_CMD_LOADHI  = %000000_11

OBJ
  'NTSC Text display object
  console : "JAM_run_con_005"
  'SPIN driver used to read the data to show the test result
  ram : "HX512KSRAM_SPIN_DRV_010"
VAR
  long status
PUB HiSpeedTest          | x, y, errors, c 

  'Start video output
  console.Init(24, 13)

  console.Print(string("Writing SRAM... "))

  ram.SRAM_InitalizeIO_S
  
  'Post-increment on both operations
  ram.SRAM_WriteControl_S(%0000_1111)

  errors := 0



  'Repeat 20 times for each free cog
  repeat c from 3 to 7
   repeat 20
    'Clear the status flag so that we know when the
    'ASM code has finished.
    status := 0
    'Write some test data to see
    'if anything new was actually written
    '(and that not too much was written)
    ram.SRAM_LoadAddr64K_S(0)
    
    repeat x from 0 to 512        
      ram.SRAM_WriteAuto_S($88)

    'Overwrite 256 bytes of the data
    'just written using the uber-fast ASM code.         

    ram.SRAM_LoadAddr64K_S(0)
    
    coginit(c, @asm_begin, @status)

    'Wait for the ASM code to signal that it's finished.
    '(The code writes number of 80MHz clock cycles taken
    'by the actual write operation)   
    repeat while status == 0

    'Verify data 
    ram.SRAM_LoadAddr64K_S(0)

    repeat x from 0 to 255 + 16
      if(x > 255)
        if(ram.SRAM_ReadAuto_S <> $88)
          errors++
          ShowError(c)
          quit
      else
        if(ram.SRAM_ReadAuto_S <> x)
          errors++
          ShowError(c)
          quit


  console.NextLine          
  console.PrintDec(errors, 3)
  console.Print(string(" of a hundred ops were errorenous"))
  
  console.NextLine
  console.Print(string("Writing 256 bytes took "))
  console.PrintDec(status, 8)
  console.NextLine
  console.PrintLn(string("clock cycles"))
  
   
  'Read the data (using the driver included)
  'and display it on the screen
  
  'Start at zero - that's where the data is loaded to.
 ' ram.SRAM_LoadAddr64K_S(0)

 ' repeat x from 0 to 255
'    ram.SRAM_WriteAuto_S($88)
  
  ram.SRAM_LoadAddr64K_S(0)

  
  'Read 256 + 32 bytes - some extra to show that no memory
  'after the first 256 bytes was written.
  y:=0
  repeat 16
    console.NextLine
    repeat 16
      x := ram.SRAM_ReadAuto_S
      if(x<>y)
        console.SetColor(7,0)
      else
        console.SetColor(0,5)
              
      console.PrintHex(x, 2)
      y++

  console.RestoreColor
        
  console.NextLine
  console.NextLine
  console.PrintLn(string("Next line should be all '88':"))
  repeat 16
      x := ram.SRAM_ReadAuto_S
      console.PrintHex(x, 2)
PUB ShowError(c) | x, y

  console.PrintDivider
  console.NextLine
  console.Print(string("Error occurred on Cog "))
  console.PrintDec(c, 1)
  console.NextLine
  


  ram.SRAM_LoadAddr64K_S(0)

  'Read 256 + 32 bytes - some extra to show that no memory
  'after the first 256 bytes was written.
  y:=0
  repeat 16
    console.NextLine
    repeat 16
      x := ram.SRAM_ReadAuto_S
      if(x<>y)
        console.SetColor(7,0)
      else
        console.SetColor(0,5)
              
      console.PrintHex(x, 2)
      y++

  console.RestoreColor
        
  console.NextLine
  console.NextLine
  console.PrintLn(string("Next line should be all '88':"))
  repeat 16
      x := ram.SRAM_ReadAuto_S
      console.PrintHex(x, 2)
console.NextLine
console.waitAnyKeyPrompt(string("--CONTINUE--"))
DAT
asm_begin

                        'Fill a buffer in COG Ram
                        'with test data (bytes 0...255)              
                        call #fill_buffer
        
                        'The SRAM Card is programmed
                        'using the SPIN driver before
                        'calling the ASM.
                        
                        'Initialize OUTPUT

                        mov outa, #0 'No accidental operations are wanted,
                                'especially with the SRAM card.                                
                        mov dira, dira_initial_value 'Note: CLOCK STROBE
                                'is not set as output yet as it would
                                'write arbitrary data to the SRAM.
                        
                        'Initialize COUNTER - Single-ended output-mode,
                        'divide PLL16X output by 4.
                        movi ctra, #%00010_101  

                        
                        or dira, dira_strobe
                        
                        'Initialize VIDEO GENERATOR
                        mov vcfg, vcfg_value
                        mov vscl, vscl_value

                        mov r0, #63  '63, not 64 - one frame (long)
                                      'is written out of the loop.                   
 
                        mov frqa, frqa_value

                        'Start timing (THIS HAS NOTHING
                        'TO DO WITH THE ACTUAL WRITING -
                        'IT JUST MEASURES HOW LONG THE
                        'WRITING TAKES!)
                         mov r1, cnt

 
                        'The first waitvid (YES, THIS IS
                        '"VISIBLE" TOO) - note that
                        'currently the video generator
                        'is pooping out arbitrary data, so
                        'let's wait until it's done with
                        'that and only then set clock as
                        'output

                        waitvid buffer, #%%0123
                        'Direct the PLL output (NON-INVERTED) to clock strobe pin 
                        movs ctra, #SRAM_STROBE 'ENABLE OUTPUT ON SRAM CLOCK STROBE  
                        
loop
                        'Output 4 "pixels" (bytes) sequentially                        
wv                      waitvid buffer+1, #%%0123
                        add wv, dest_one 'Next long (increment COLOR pointer)
                        djnz r0, #loop
                        'Wait, don't cut output too early.
                        nop
                        'All as input - no clock output no more,
                        'so that exactly 256 bytes'd be written.
                        movs ctra, #0 'DISABLE OUTPUT ON SRAM CLOCK STROBE  


                        'Stop timing -                        
                        'Substract CNT on start from
                        'current CNT to get the number
                        'of clock cycles taken.                                                

                        mov r0, cnt
                        sub r0, r1

                        'Tell SPIN code that it's all written...
                        wrlong r0, par

                        'Die (Stop this cog)
                        cogid r0
                        cogstop r0                        


'Fill the 64-long (256-byte) buffer with 1-byte numbers 0...256
fill_buffer             mov r0, #64

:fill_buffer_lp         mov buffer, buffer_acc
                        add :fill_buffer_lp, dest_one
                        add buffer_acc, buffer_increment
                        djnz r0, #:fill_buffer_lp                        

fill_buffer_ret         ret


r0                      long    0
r1                      long    0       

vcfg_value              long    %0_01_1__0_0_0_000_00__00000000_010_01111__1111                  

                        'Frequency of 15Mhz - the MIPS clock speed of the COG.
frqa_value              long    ((1 << 31) >> 3) / 20 * 15 


vscl_value              long    (1 << 12) | 4       

dira_initial_value      long    (%1111_1111 << 16) | (1<<SRAM_CTRL_0) | (1 << SRAM_CTRL_1)
dira_strobe             long    1<<SRAM_STROBE




dest_one                long 1<<9

'These constants are used to generate the test data.
buffer_acc              long $00_01_02_03
buffer_increment        long $04_04_04_04


buffer                  res 64